const express = require('express');
const router = express.Router();
const connection = require('./db');
const languageMessages = require('./language_messages');
const multer = require('multer');
const path = require('path');
const crypto = require('crypto');
const moment = require('moment-timezone');
// Import mail functions and site configuration
const { sendMail, AddVIPAccount } = require('./mailfunction.js');
const { app_name, app_logo, ios_url, android_url, admin_url,main_admin_url } = require('./site_config');

// Set up Multer for file uploads
const storage = multer.diskStorage({
    destination: function (req, file, cb) {
        cb(null, '../webservice/images'); // Save uploaded files to the 'webservice/images' directory
    },
    filename: function (req, file, cb) {
        const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
        cb(null, 'user_image-' + uniqueSuffix + path.extname(file.originalname));
    }
});

const upload = multer({ storage: storage });

const initController = () => {
    router.post('/add_vip_user', upload.single('image'), async (req, res) => {
        try {
            const { username, mobile, email, password, role } = req.body;
            const user_type = 2;
            const profile_complete = 1;

            // Check if required fields are provided
            if (!username || !password) {
                return res.status(400).json({ success: false, msg: languageMessages.allFieldsRequired[0] });
            }

            // Get the filename of the uploaded image
            let image = '';
            if (req.file) {
                image = req.file.filename;
            } else {
                image = ''; // or any other default value
            }
            const currentTime= moment.tz('America/New_York').format('YYYY-MM-DD HH:mm:ss');
            // Wrap the query inside a Promise for async/await usage
            const checkUsernameQuery = 'SELECT user_id FROM user_master WHERE username = ? AND delete_flag = 0 AND profile_complete = 1 and user_type=2';
            const usernameResults = await queryAsync(checkUsernameQuery, [username]);

            if (usernameResults.length > 0) {
                return res.status(200).json({ success: false, msg: languageMessages.msgUserNameExxistError[0] });
                //return res.status(400).json({ success: false, msg: languageMessages.msgUserNameExxistError[0] });
            }

            

            if(email){
                // Check if email already exists
                const checkEmailQuery = 'SELECT user_id FROM user_master WHERE email = ? AND delete_flag = 0 AND profile_complete = 1';
                const emailResults = await queryAsync(checkEmailQuery, [email]);

                if (emailResults.length > 0) {
                    return res.status(200).json({ success: false, msg: languageMessages.msgEmailError[0] });
                    //return res.status(400).json({ success: false, msg: languageMessages.msgEmailError[0] });
                }
            }
            // Add user details to the database
            const hashedPassword = md5(password);
            const insertQuery = 'INSERT INTO user_master (username, mobile, email, password, user_type, role, image, password_new, profile_complete, createtime, updatetime) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)';
            const insertResults = await queryAsync(insertQuery, [username, mobile, email, hashedPassword, user_type, role, image, password, profile_complete,currentTime,currentTime]);
            
            if(email){
                // Send email notification
                const fromName = app_name;
                const subject = 'Account Credentials';
                let mailContent = '';
                if (role == 1) {
                    mailContent = `
                        <p>Hello ${username},</p>
                        <p>Your account has been created by administration.</p>
                        <p>Username: ${username}</p>
                        <p>Password: ${password}</p>
                        <p>iOS App URL: ${ios_url}</p>
                        <p>Android App URL: ${android_url}</p>
                        <p>Admin URL: ${admin_url}</p>`;
                } else {
                    mailContent = `
                        <p>Hello ${username},</p>
                        <p>Your account has been created by administration.</p>
                        <p>Username: ${username}</p>
                        <p>Password: ${password}</p>
                        <p>Admin URL: ${main_admin_url}</p>`;
                }

                
                const mailBody = AddVIPAccount({ email, fromName, subject, mailContent, app_logo, username });
                await sendMail(email, fromName, subject, mailBody);
            }

            return res.status(200).json({ success: true, msg: languageMessages.msgVIPUserAddedSuccess[0] });
        } catch (error) {
            console.error('Error handling request:', error);
            return res.status(500).json({ success: false, msg: languageMessages.internalServerError[0] });
        }
    });

    return router;
};

// MD5 hashing function
function md5(input) {
    return crypto.createHash('md5').update(input).digest('hex');
}

// Helper function to execute queries with async/await
function queryAsync(query, values) {
    return new Promise((resolve, reject) => {
        connection.query(query, values, (error, results) => {
            if (error) {
                reject(error);
            } else {
                resolve(results);
            }
        });
    });
}

module.exports = initController;