// Import necessary modules
const express = require('express');
const multer = require('multer');
const path = require('path');
const router = express.Router();
const connection = require('./db');
const languageMessages = require('./language_messages'); // Import language messages
const moment = require('moment-timezone');

// Set up Multer for file uploads
const storage = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, '../server/webservice/images'); // Save uploaded files to the 'webservice/images' directory (Make sure this path is correct)
  },
  filename: function (req, file, cb) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
    cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));
  }
});

const upload = multer({ storage: storage });

// Route to handle adding keywords
const initController = () => {
    router.post('/create_subadmin_post', upload.array('files', 8), async (req, res) => {
      try {
        // Validate inputs
        const { caption, address, post_type, user_id, latitude, longitude, remaining_fund } = req.body;
  
        if (!address || !post_type || !user_id || !latitude || !longitude) {
          return res.status(400).json({ success: false, msg: languageMessages.allFieldsRequired[0], 'req_body': req.body });
        }
  
        // Check if user has set a permanent location
        const selectQuery = 'SELECT currect_location_permanent, current_location, current_lattitude, current_longitude FROM user_master WHERE user_id = ?';
        connection.query(selectQuery, [user_id], (selectError, selectResults) => {
          if (selectError) {
            console.error('Error selecting user:', selectError);
            return res.status(200).json({ success: false, msg: languageMessages.internalServerError[0], 'selectError': selectError });
          }
  
          if (selectResults.length > 0) {
            const { currect_location_permanent, current_location, current_lattitude, current_longitude } = selectResults[0];
            
            // If permanent location is set, use it instead of the provided address, latitude, and longitude
            const finalAddress = currect_location_permanent === 1 ? current_location : address;
            const finalLatitude = currect_location_permanent === 1 ? current_lattitude : latitude;
            const finalLongitude = currect_location_permanent === 1 ? current_longitude : longitude;
  
            // Upload images
            const compressedFileNames = req.files.map(file => file.filename);
            const created_by = 1;
            const currentTime= moment.tz('America/New_York').format('YYYY-MM-DD HH:mm:ss');
            // Add keyword to the database
            const insertQuery = 'INSERT INTO post_master (post_type, created_by, user_id, caption, location, latitude, longitude, createtime, updatetime) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)';
            connection.query(insertQuery, [post_type, created_by, user_id, caption, finalAddress, finalLatitude, finalLongitude, currentTime, currentTime], (insertError, insertResults) => {
              if (insertError) {
                console.error('Error inserting post:', insertError);
                return res.status(500).json({ success: false, msg: languageMessages.msgPostCreateError[0], 'insertError66': insertError, 'req_body': req.body });
              }
  
              // Insert image file names into the database
              const postId = insertResults.insertId;
              const imageInsertQuery = 'INSERT INTO post_image_master (post_id, image, createtime, updatetime) VALUES (?, ?, NOW(), NOW())';
              const imageInsertPromises = [];
  
              for (const fileName of compressedFileNames) {
                const promise = new Promise((resolve, reject) => {
                  connection.query(imageInsertQuery, [postId, fileName], (imageInsertError, imageInsertResults) => {
                    if (imageInsertError) {
                      console.error('Error inserting image:', imageInsertError);
                      reject(imageInsertError);
                    } else {
                      console.log('Image inserted:', fileName);
                      resolve(imageInsertResults);
                    }
                  });
                });
                imageInsertPromises.push(promise);
              }
  
              Promise.all(imageInsertPromises)
                .then(() => {
                  // Update user_master table here
                  const updateQuery = 'UPDATE user_master SET account_fund_balance = ? WHERE user_id = ?';
                  connection.query(updateQuery, [remaining_fund, user_id], (updateError, updateResults) => {
                    if (updateError) {
                      console.error('Error updating user_master:', updateError);
                      return res.status(500).json({ success: false, msg: languageMessages.msgUserUpdateError[0], 'updateError': updateError });
                    }
                    console.log('User updated:', updateResults);
                    return res.status(200).json({ success: true, msg: languageMessages.msgPostCreateSuccess[0], post_type: post_type });
                  });
                })
                .catch(error => {
                  console.error('Error inserting images:', error);
                  return res.status(500).json({ success: false, msg: languageMessages.msgPostCreateError[0] });
                });
            });
          } else {
            return res.status(404).json({ success: false, msg: languageMessages.userNotFound[0], 'req_body': req.body });
          }
        });
      } catch (error) {
        console.error('Error handling request:', error);
        return res.status(200).json({ success: false, msg: languageMessages.internalServerError[0], 'error': error });
      }
    });
    return router;
  };
  


module.exports = initController;
