const express = require('express');
const router = express.Router();
const connection = require('./db');
const languageMessages = require('./language_messages');
const crypto = require('crypto');
const multer = require('multer');
const path = require('path');
const moment = require('moment-timezone');

// Set up Multer for file uploads
const storage = multer.diskStorage({
    destination: function (req, file, cb) {
        cb(null, path.resolve(__dirname, '../webservice/images')); // Absolute path to the 'images' directory
    },
    filename: function (req, file, cb) {
        const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
        cb(null, 'user_image-' + uniqueSuffix + path.extname(file.originalname));
    }
});

const upload = multer({ 
    storage: storage,
    limits: { fileSize: 5 * 1024 * 1024 } // Limit file size to 5MB
});

const initController = () => {
    router.post('/edit_guest_user', upload.single('image'), async (req, res) => {
        try {
            const { user_id, username1, mobile1, email1, bio,note_by_admin } = req.body;

            // Check if required fields are provided
            if (!user_id || !username1) {
                return res.status(400).json({ success: false, msg: languageMessages.allFieldsRequired[0] });
            }

            // Provide default values for empty fields
            const defaultValues = {
                bio: '',
                address: '',
                mobile1: '',
                email1: '',
                note_by_admin:'',
            };
            
            // Merge default values with the request body
            const requestData = { ...req.body, ...defaultValues };
            //const hashedPassword = md5(password1);
            const currentTime= moment.tz('America/New_York').format('YYYY-MM-DD HH:mm:ss');
            // Check if email is provided and validate it
            if (username1) {
                // Check if email already exists
                const checkEmailQuery = 'SELECT user_id FROM user_master WHERE username = ? AND delete_flag = 0 AND profile_complete = 1 and user_id != ? and user_type=1';
                connection.query(checkEmailQuery, [username1, user_id], (usernameError, usernameResults) => {
                    if (usernameError) {
                        console.error('Error checking email:', usernameError);
                        return res.status(500).json({ success: false, msg: languageMessages.internalServerError[0], error: usernameError });
                    }
                    if (usernameResults.length > 0) {
                        return res.status(200).json({ success: false, msg: languageMessages.msgUserNameExxistError[0] });
                        //return res.status(400).json({ success: false, msg: languageMessages.msgUserNameExxistError[0] });
                    }
                    // Continue with the rest of the process
                    updateUser();
                });
            } else {
                // Continue with the rest of the process if email is not provided
                updateUser();
            }

            // Check if email is provided and validate it
            if (email1) {
                // Check if email already exists
                const checkEmailQuery = 'SELECT user_id FROM user_master WHERE email = ? AND delete_flag = 0 AND profile_complete = 1 and user_id != ?';
                connection.query(checkEmailQuery, [email1, user_id], (emailError, emailResults) => {
                    if (emailError) {
                        console.error('Error checking email:', emailError);
                        return res.status(500).json({ success: false, msg: languageMessages.internalServerError[0], error: emailError });
                    }
                    if (emailResults.length > 0) {
                        return res.status(200).json({ success: false, msg: languageMessages.msgEmailError[0] });
                    }
                    // Continue with the rest of the process
                    updateUser();
                });
            } else {
                // Continue with the rest of the process if email is not provided
                updateUser();
            }

            function updateUser() {
                // Update user details in the database
                let updateQuery = `UPDATE user_master SET username = ?, mobile = ?, email = ?,about = ?,note_by_admin=?, updatetime = ?`;
                let queryValues = [username1, mobile1, email1,bio,note_by_admin,currentTime];

                // Check if an image was uploaded
                if (req.file) {
                    // Include image update in the query
                    updateQuery += ', image = ?';
                    queryValues.push(req.file.filename);
                }

                updateQuery += ' WHERE user_id = ?';
                queryValues.push(user_id);

                // Execute the database update query
                connection.query(updateQuery, queryValues, (updateError, updateResults) => {
                    if (updateError) {
                        console.error('Error updating user:', updateError);
                        return res.status(200).json({ success: false, msg: languageMessages.msgGuestUserupdatedError[0],error: updateError});
                    }
                    return res.status(200).json({ success: true, msg: languageMessages.msgGuestUserupdatedSuccess[0] });
                });
            }
        } catch (error) {
            console.error('Error handling request:', error);
            return res.status(500).json({ success: false, msg: languageMessages.internalServerError[0], error: error });
        }
    });

    return router;
};

// MD5 hashing function
function md5(input) {
    return crypto.createHash('md5').update(input).digest('hex');
}

module.exports = initController;
