const express = require('express');
const router = express.Router();
const connection = require('./db');
const languageMessages = require('./language_messages');
const crypto = require('crypto');
const multer = require('multer');
const path = require('path');
const moment = require('moment-timezone');

// Set up Multer for file uploads
const storage = multer.diskStorage({
    destination: function (req, file, cb) {
        cb(null, path.resolve(__dirname, '../webservice/images')); // Absolute path to the 'images' directory
    },
    filename: function (req, file, cb) {
        const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
        cb(null, 'user_image-' + uniqueSuffix + path.extname(file.originalname));
    }
});

const upload = multer({ 
    storage: storage,
    limits: { fileSize: 5 * 1024 * 1024 } // Limit file size to 5MB
});

const initController = () => {
    router.post('/edit_vip_user', upload.single('image'), async (req, res) => {
        try {
            const { user_id, username1, mobile1, email1, password1, bio, address, account_currency , note_by_admin, active_flag, latitude, longitude,add_account_fund_balance,currect_location_permanent} = req.body;

            let account_fund_balance = req.body.account_fund_balance;
            
            // Check if required fields are provided
            if (!user_id || !username1 || !password1) {
                return res.status(400).json({ success: false, msg: languageMessages.allFieldsRequired[0] });
            }

            let new_account_fund_balance = 0; // Default value

            // Check if account_fund_balance is provided and not 0
            if (account_fund_balance !== undefined && account_fund_balance !== null && account_fund_balance !== '' && account_fund_balance !== 0) {
                // Parse the provided account_fund_balance as float
                new_account_fund_balance = parseFloat(account_fund_balance);
            }

            // Check if add_account_fund_balance is provided and not 0
            if (add_account_fund_balance !== undefined && add_account_fund_balance !== null && add_account_fund_balance !== '' && add_account_fund_balance !== 0) {
                // Parse the additional balance as float
                let additionalBalance = parseFloat(add_account_fund_balance);

                // Split the original balance into its integer and decimal parts
                let integerPart = Math.floor(new_account_fund_balance);
                let decimalPart = new_account_fund_balance - integerPart;

                // Add the additional balance to the integer part
                integerPart += additionalBalance;

                // Update the new account fund balance
                new_account_fund_balance = integerPart + decimalPart;
            }

            // Provide default values for empty fields
            const defaultValues = {
                bio: '',
                address: '',
                //new_account_fund_balance: 0 // Assuming this field is numeric, use 0 as the default value
            };

            // Merge default values with the request body
            const requestData = { ...req.body, ...defaultValues };
            const hashedPassword = md5(password1);
            const currentTime= moment.tz('America/New_York').format('YYYY-MM-DD HH:mm:ss');

            // Check if username already exists
            const checkUsernameQuery = 'SELECT user_id FROM user_master WHERE username = ? AND delete_flag = 0 AND profile_complete = 1 and user_id!=?';
            connection.query(checkUsernameQuery, [username1, user_id], (usernameError, usernameResults) => {
                if (usernameError) {
                    console.error('Error checking username:', usernameError);
                    return res.status(500).json({ success: false, msg: languageMessages.internalServerError[0], error: usernameError });
                }
                if (usernameResults.length > 0) {
                    return res.status(200).json({ success: false, msg: languageMessages.msgUserNameExxistError[0] });
                }

                if (email1) {
                    // Check if email already exists
                    const checkEmailQuery = 'SELECT user_id FROM user_master WHERE email = ? AND delete_flag = 0 AND profile_complete = 1 and user_id!=?';
                    connection.query(checkEmailQuery, [email1, user_id], (emailError, emailResults) => {
                        if (emailError) {
                            console.error('Error checking email:', emailError);
                            return res.status(500).json({ success: false, msg: languageMessages.internalServerError[0], error: emailError });
                        }
                        if (emailResults.length > 0) {
                            return res.status(200).json({ success: false, msg: languageMessages.msgEmailError[0] });
                        }

                        // Update user details in the database
                        let updateQuery = `UPDATE user_master SET username = ?, mobile = ?, email = ?, password = ?, password_new = ?, current_location = ?, account_currency = ?, account_fund_balance = ?, note_by_admin = ?, active_flag = ?, about = ?, current_lattitude=?, current_longitude=?, updatetime = ?,currect_location_permanent=?`;
                        let queryValues = [username1, mobile1, email1, hashedPassword, password1, address, account_currency, account_fund_balance, note_by_admin, active_flag, bio, latitude, longitude, currentTime,currect_location_permanent];

                        // Check if an image was uploaded
                        if (req.file) {
                            // Include image update in the query
                            updateQuery += ', image = ?';
                            queryValues.push(req.file.filename);
                        }

                        updateQuery += ' WHERE user_id = ?';
                        queryValues.push(user_id);

                        // Execute the database update query
                        connection.query(updateQuery, queryValues, (updateError, updateResults) => {
                            if (updateError) {
                                console.error('Error updating user:', updateError);
                                return res.status(500).json({ success: false, msg: languageMessages.msgVIPUserupdatedError[0], error: updateError });
                            }
                            return res.status(200).json({ success: true, msg: languageMessages.msgVIPUserupdatedSuccess[0],new_account_fund_balance:new_account_fund_balance });
                        });
                    });
                } else {
                    // Update user details in the database when email is not provided
                    let updateQuery = `UPDATE user_master SET username = ?, mobile = ?, email = ?, password = ?, password_new = ?, current_location = ?, account_currency = ?, account_fund_balance = ?, note_by_admin = ?, active_flag = ?, about = ?, current_lattitude=?, current_longitude=?, updatetime = ?,currect_location_permanent=?`;
                    let queryValues = [username1, mobile1, email1,hashedPassword,password1, address, account_currency, account_fund_balance, note_by_admin, active_flag, bio, latitude, longitude, currentTime,currect_location_permanent];

                    // Check if an image was uploaded
                    if (req.file) {
                        // Include image update in the query
                        updateQuery += ', image = ?';
                        queryValues.push(req.file.filename);
                    }

                    updateQuery += ' WHERE user_id = ?';
                    queryValues.push(user_id);

                    // Execute the database update query
                    connection.query(updateQuery, queryValues, (updateError, updateResults) => {
                        if (updateError) {
                            console.error('Error updating user:', updateError);
                            return res.status(500).json({ success: false, msg: languageMessages.msgVIPUserupdatedError[0], error: updateError });
                        }
                        return res.status(200).json({ success: true, msg: languageMessages.msgVIPUserupdatedSuccess[0],new_account_fund_balance:new_account_fund_balance });
                    });
                }
            });
        } catch (error) {
            console.error('Error handling request:', error);
            return res.status(500).json({ success: false, msg: languageMessages.internalServerError[0], error: error });
        }
    });

    return router;
};

// MD5 hashing function
function md5(input) {
    return crypto.createHash('md5').update(input).digest('hex');
}

module.exports = initController;
