// Import necessary modules
const express = require('express');
const multer = require('multer');
const path = require('path');
const router = express.Router();
const connection = require('./db');
const languageMessages = require('./language_messages');

// Set up Multer for file uploads
const storage = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, path.join(__dirname, '../webservice/images')); // Use path.join for consistent file path resolution
  },
  filename: function (req, file, cb) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
    cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));
  }
});
const upload = multer({
  storage: storage,
  limits: { fileSize: 5 * 1024 * 1024 } // Limit file size to 5MB
});

// Route to handle editing profile
const initController = () => {
  router.post('/edit_profile', upload.single('image'), async (req, res) => {
    try {
      // Validate inputs
      const user_id = req.body.user_id;
      const email = req.body.email;
      const username = req.body.username;
      const image = req.file; // Access the uploaded image file

      // Function to validate username
      const validateUsername = (username, user_id) => {
        return new Promise((resolve, reject) => {
          if (username) {
            const checkUsernameQuery = 'SELECT user_id FROM user_master WHERE username = ? AND delete_flag = 0 AND profile_complete = 1 AND user_id != ? AND user_type IN (0, 2) AND role = 0';
            connection.query(checkUsernameQuery, [username, user_id], (err, results) => {
              if (err) {
                console.error('Error checking username:', err);
                return reject(err);
              }
              if (results.length > 0) {
                return resolve({ isValid: false, msg: languageMessages.msgUserNameExxistError[0] });
              }
              resolve({ isValid: true });
            });
          } else {
            resolve({ isValid: true }); // Continue if username is not provided
          }
        });
      };

      // Function to validate email
      const validateEmail = (email, user_id) => {
        return new Promise((resolve, reject) => {
          if (email) {
            const checkEmailQuery = 'SELECT user_id FROM user_master WHERE email = ? AND delete_flag = 0 AND profile_complete = 1 AND user_id != ?';
            connection.query(checkEmailQuery, [email, user_id], (err, results) => {
              if (err) {
                console.error('Error checking email:', err);
                return reject(err);
              }
              if (results.length > 0) {
                return resolve({ isValid: false, msg: languageMessages.msgEmailError[0] });
              }
              resolve({ isValid: true });
            });
          } else {
            resolve({ isValid: true }); // Continue if email is not provided
          }
        });
      };

      // Validate email and username
      const isValidUsername = await validateUsername(username, user_id);
      if (!isValidUsername.isValid) {
        return res.status(400).json({ success: false, msg: isValidUsername.msg });
      }

      const isValidEmail = await validateEmail(email, user_id);
      if (!isValidEmail.isValid) {
        return res.status(400).json({ success: false, msg: isValidEmail.msg });
      }

      // Construct the query and parameters based on whether an image is provided or not
      let query = 'UPDATE user_master SET email = ?, username = ?, updatetime = NOW()';
      const params = [email, username];

      if (image) {
        query += ', image = ?';
        params.push(image.filename);
      }

      query += ' WHERE user_id = ?';
      params.push(user_id);

      // Update user profile in the database
      connection.query(query, params, (error, results) => {
        if (error) {
          console.error('Error updating user profile:', error);
          return res.status(500).json({ success: false, msg: languageMessages.msgProfileUpdateError[0], error });
        }
        return res.status(200).json({ success: true, msg: languageMessages.msgProfileUpdateSuccess[0] });
      });
    } catch (error) {
      console.error('Error updating user profile:', error);
      return res.status(500).json({ success: false, msg: languageMessages.internalServerError[0], error });
    }
  });

  return router;
};

module.exports = initController;
