const express = require('express');
const multer = require('multer');
const path = require('path');
const router = express.Router();
const connection = require('./db');
const languageMessages = require('./language_messages');
const { currentTime } = require('./site_config');

// Set up Multer for file uploads
const storage = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, path.join(__dirname, '../webservice/images')); // Save uploaded files to the 'images' directory
  },
  filename: function (req, file, cb) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
    cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));
  }
});

const upload = multer({
  storage: storage,
  limits: { fileSize: 200 * 1024 * 1024 } // Set limit to 200MB
});

const initController = () => {
  router.post('/update_website_content', upload.fields([
    { name: 'logo', maxCount: 1 }, 
    { name: 'document', maxCount: 1 }, 
    { name: 'apk', maxCount: 1 },
    { name: 'promotional_image', maxCount: 1 },
    { name: 'promotional_image_second', maxCount: 1 },
    { name: 'images[]', maxCount: 12 },
    { name: 'screenshot[]', maxCount: 10 },
    { name: 'images_edit1', maxCount: 1 },
    { name: 'images_edit2', maxCount: 1 },
    { name: 'images_edit3', maxCount: 1 },
    { name: 'images_edit4', maxCount: 1 },
    { name: 'images_edit5', maxCount: 1 },
    { name: 'images_edit6', maxCount: 1 },
    { name: 'images_edit7', maxCount: 1 },
    { name: 'images_edit8', maxCount: 1 },
    { name: 'images_edit9', maxCount: 1 },
    { name: 'images_edit10', maxCount: 1 },
    { name: 'images_edit11', maxCount: 1 },
    { name: 'images_edit12', maxCount: 1 }
  ]), async (req, res) => {
    try {
      const { website_content_id, about_us, about_us_chinese, contact_us, contact_us_chinese, promotional_file, promotional_file_chinese, j, m,promotional_file_second, promotional_file_chinese_second,facebook_url,twitter_url,instagram_url,pinterest_url,linkedin_url } = req.body;
      
      // Check if required fields are present
      if (!website_content_id || !about_us || !about_us_chinese || !contact_us || !contact_us_chinese || !promotional_file || !promotional_file_chinese ||!promotional_file_second ||!promotional_file_chinese_second ||!facebook_url ||!twitter_url ||!instagram_url ||!pinterest_url ||!linkedin_url){
        return res.status(400).json({ success: false, msg: "Content ID and content are required" });
      }

      // Check if logo file, document file, APK file are provided
      const logo = req.files['logo'] ? req.files['logo'][0] : null;
      const documentFile = req.files['document'] ? req.files['document'][0] : null;
      const apkFile = req.files['apk'] ? req.files['apk'][0] : null;
      const images = req.files['images[]'] || [];
      const screenshot = req.files['screenshot[]'] || [];
      const promotional_image = req.files['promotional_image'] ? req.files['promotional_image'][0] : null;
      const promotional_image_second = req.files['promotional_image_second'] ? req.files['promotional_image_second'][0] : null;
      
      // Construct the query and parameters
      let query = 'UPDATE website_content_master SET promotional_file = ?, promotional_file_chinese = ?, about_us = ?, about_us_chinese = ?, contact_us = ?, contact_us_chinese = ?, updatetime = ?,promotional_file_second=?,promotional_file_chinese_second=?,facebook_url=?,twitter_url=?,instagram_url=?,pinterest_url=?,linkedin_url=?';
      const params = [promotional_file, promotional_file_chinese, about_us, about_us_chinese, contact_us, contact_us_chinese, currentTime,promotional_file_second,promotional_file_chinese_second,facebook_url,twitter_url,instagram_url,pinterest_url,linkedin_url];

      // Add logo update to the query and parameters if logo is provided
      if (logo) {
        query += ', logo = ?';
        params.push(logo.filename);
      }

      // Add document update to the query and parameters if document is provided
      if (documentFile) {
        query += ', document_file = ?';
        params.push(documentFile.filename);
      }

      // Add APK update to the query and parameters if APK is provided
      if (apkFile) {
        query += ', apk_file = ?';
        params.push(apkFile.filename);
      }

      // Add promotional_image update to the query and parameters if promotional_image is provided
      if (promotional_image) {
        query += ', promotional_image = ?';
        params.push(promotional_image.filename);
      }

      // Add promotional_image_second update to the query and parameters if promotional_image_second is provided
      if (promotional_image_second) {
        query += ', promotional_image_second = ?';
        params.push(promotional_image_second.filename);
      }

      query += ' WHERE website_content_id = ?';
      params.push(website_content_id);

      // Perform the database update operation
      connection.query(query, params, async (error, result) => {
        if (error) {
          console.error('Error updating content:', error);
          return res.status(500).json({ success: false, msg: languageMessages.internalServerError[0], error: error });
        }

        if (result.affectedRows > 0) {
          // Content updated successfully
          const blogEntries = [];
          if (j) {
            // Process each image uploaded
            for (let i = 1; i <= j; i++) {
              const image = images[i - 1]?.filename; // Adjust index to start from 0
              const text = req.body[`text_add${i}`];
              const text_chinese = req.body[`text_add_chinese${i}`];
              // Check if all required data is available
              if (image && text && text_chinese) {
                try {
                  // Construct and execute query to insert blog entry into blog_master table
                  const blogQuery = 'INSERT INTO website_blog_master (image, text_eng, text_chinese, createtime, updatetime) VALUES (?, ?, ?, ?, ?)';
                  const blogParams = [image, text, text_chinese, currentTime, currentTime];
                  const blogResult = await connection.query(blogQuery, blogParams);
                  blogEntries.push(blogResult.insertId);
                } catch (blogError) {
                  console.error('Error inserting blog entry:', blogError);
                  // Handle blog insertion error
                }
              }
            }
          }

          if(m){
            // Update existing blog entries
            for (let i = 1; i <= m; i++) {
                const text_edit = req.body[`text_edit${i}`];
                const text_edit_chinese = req.body[`text_edit_chinese${i}`];
                const website_blog_id = req.body[`website_blog_id${i}`];
                //const images_edit = req.files[`images_edit${i}`]; // Check if images_edit exists
                const images_edit = req.files[`images_edit${i}`] ? req.files[`images_edit${i}`][0] : null;
                
                // Construct the query and parameters for updating blog entry
                let blogQuery, blogParams;
                if (images_edit) {
                    // If images_edit exists and is not empty, update with image
                    const edit_image = images_edit.filename;
                    blogQuery = 'UPDATE website_blog_master SET image=?, text_eng=?, text_chinese=?, updatetime=? WHERE website_blog_id=?';
                    blogParams = [edit_image, text_edit, text_edit_chinese, currentTime, website_blog_id];
                } else {
                    // If images_edit is empty or undefined, update without image
                    blogQuery = 'UPDATE website_blog_master SET text_eng=?, text_chinese=?, updatetime=? WHERE website_blog_id=?';
                    blogParams = [text_edit, text_edit_chinese, currentTime, website_blog_id];
                }
        
                try {
                    await connection.query(blogQuery, blogParams);
                } catch (blogError) {
                    console.error('Error updating blog entry:', blogError);
                    // Handle blog update error
                }
            }
          }

          
          if (screenshot.length > 0) {
            // Process each image uploaded
            for (let i = 0; i < screenshot.length; i++) {
              const screenshot_get = screenshot[i]?.filename; // Adjust index to start from 0

              // Check if all required data is available
              if (screenshot_get) {
                try {
                  // Construct and execute query to insert blog entry into blog_master table
                  const screenshotQuery = 'INSERT INTO website_screenshot_master (screenshot_image, createtime, updatetime) VALUES (?, ?, ?)';
                  const screenshotParams = [screenshot_get, currentTime, currentTime];
                  const [screenshotResult] = await connection.query(screenshotQuery, screenshotParams);
                  screenshotEntries.push(screenshotResult.insertId);
                } catch (screenshotError) {
                  console.error('Error inserting screenshot entry:', screenshotError);
                  // Handle blog insertion error
                }
              }
            }
          }

        

          return res.json({ success: true, msg: languageMessages.websiteContentUpdateSuccessfully[0]});
        } else {
          // Content with the specified content_id not found
          return res.status(404).json({ success: false, msg: languageMessages.contentNotFound[0] });
        }
      });
    } catch (error) {
      console.error('Error updating content:', error);
      res.status(500).json({ success: false, msg: languageMessages.internalServerError[0], error: error });
    }
  });

  return router;
};

module.exports = initController;
